@@ -56,7 +56,9 @@ where
     #[emitter]
     fn emit_qualified_rule(&mut self, n: &QualifiedRule) -> Result {
         emit!(self, n.prelude);
-        space!(self);
+        if !self.config.minify {
+            space!(self);
+        }
         emit!(self, n.block);
     }
 
@@ -158,7 +160,7 @@ where
     }
 
     #[emitter]
-    fn emit_keyfram_block(&mut self, n: &KeyframeBlock) -> Result {
+    fn emit_keyframe_block(&mut self, n: &KeyframeBlock) -> Result {
         self.emit_list(&n.selector, ListFormat::CommaDelimited)?;
 
         space!(self);
@@ -738,7 +740,30 @@ where
 
     #[emitter]
     fn emit_complex_selector(&mut self, n: &ComplexSelector) -> Result {
-        self.emit_list(&n.children, ListFormat::SpaceDelimited)?;
+        let mut need_space = false;
+        for (idx, node) in n.children.iter().enumerate() {
+            match node {
+                ComplexSelectorChildren::Combinator(..) => {
+                    need_space = false;
+                }
+                _ => {}
+            }
+
+            if idx != 0 && need_space {
+                need_space = false;
+
+                self.wr.write_space()?;
+            }
+
+            match node {
+                ComplexSelectorChildren::CompoundSelector(..) => {
+                    need_space = true;
+                }
+                _ => {}
+            }
+
+            emit!(self, node)
+        }
     }
 
     #[emitter]
